#####################################################################################################
#												    #
#	This archive is part of the free distribution of data and statistical software code for	    #
#	"Generalized Linear Models: A Unified Approach", Jeff Gill, Sage QASS Series.  You are	    #
#	free to use, modify, distribute, publish, etc. provided attribution.  Please forward 	    #
#       bugs, complaints, comments, and useful changes to: jgill@latte.harvard.edu.  		    #
#												    #
#####################################################################################################

###################################### SETUP ########################################################
committee.dat <- read.table("committee.dat", header=T,col.names=1)
attach(committee.dat)
#####################################################################################################

###################################### MODEL ########################################################
committee.out <- glm.nb(BILLS104 ~ SIZE + SUBS ttee.out <- glm.nb(BILLS104 ~ SIZE + 
	SUBS * (log(STAFF)) + PRESTIGE + BILLS103)
summary.glm(committee.out)
diag.plot(committee.out)
sort(abs( eigen(-glm.vc(committee.out))$values^(-1) ))

committee.se <- sqrt(diag(committee.covmat))
round( cbind(
        committee.out$coefficients, committee.se,
        committee.out$coefficients - committee.se*qt(0.975,committee.out$df.residual),
        committee.out$coefficients + committee.se*qt(0.975,committee.out$df.residual)
),7)
#####################################################################################################

################################# RESIDUALS GRAPH  ##################################################
z.matrix <- matrix(0,200,200)
for(i in 1:200)  {
        for(j in 1:200)  {
                if(j < 70)    z.matrix[i,j] <- 1
                if(j < 40)    z.matrix[i,j] <- 2
                if(j < 10)    z.matrix[i,j] <- 3
                if(j == 1)    z.matrix[i,j] <- 3.001
                if(j > 130)   z.matrix[i,j] <- 1
                if(j > 160)   z.matrix[i,j] <- 2
                if(j > 190)   z.matrix[i,j] <- 3
                if(j == 200)  z.matrix[i,j] <- 3.001
        }
}

resp <- resid(committee.out,type="response")
pears <- resid(committee.out,type="pearson")
working <- resid(committee.out,type="working")
devs <- resid(committee.out,type="deviance")

r1 <- committee.out$y/max(committee.out$y)
r2 <- committee.out$fitted.values/max(committee.out$fitted.values)
phi.r1 <- cs.string[ceiling(r1*1000)+1,2]
phi.r2 <- cs.string[ceiling(r2*1000)+1,2]
anscombe <- (phi.r1^(2/3) - phi.r2^(2/3))/( (r2^(1/6)) * ((1-r2)^(1/6)) )
anscombe[4] <- devs[4]

par(mfrow=c(4,1),mar=c(0,1,0,1))
plot(pears[order(BILLS104)],type="l",ylim=c(-2,2),xaxt="n")
plot(working[order(BILLS104)],type="l",ylim=c(-2,2),xaxt="n")
plot(devs[order(BILLS104)],type="l",ylim=c(-2,2),xaxt="n")
plot(anscombe[order(BILLS104)],type="l",ylim=c(-2,2),xaxt="n")

postscript("glm.fig5.ps")
par(mfrow=c(1,1),oma=c(3,3,1,1))
image(seq(0,50,length=200),seq(-2000,2000,length=200),z.matrix,xlim=c(0,50),ylim=c(-2000,2000),
	xaxt="n",yaxt="n",xlab="",ylab="")
points(seq(1,50,length=20),(2000/3)*pears[order(BILLS104)],pch=15)
lines(seq(1,50,length=20),(2000/3)*devs[order(BILLS104)],type="h")
mtext(line=3,side=1,cex=2,"Order of Fitted Outcome Variable")
mtext(line=2,side=2,cex=2,"Residual Effect")
abline(0,0)
dev.off()
#####################################################################################################

########################### COVARIANCE ANALYSIS #####################################################
p1 <- 1:committee.out$rank
committee.covmat <- chol2inv(committee.out$qr$qr[p1, p1, drop = FALSE])*1.494362
committee.se <- sqrt(diag(committee.covmat))[-1]
committee.coef <- committee.out$coef
committee.mat <- cbind(BILLS104, SIZE, SUBS, log(STAFF), PRESTIGE, BILLS103, SUBS*log(STAFF))
#####################################################################################################


