#####################################################################################################
#												    #
#	This archive is part of the free distribution of data and statistical software code for	    #
#	"Generalized Linear Models: A Unified Approach", Jeff Gill, Sage QASS Series.  You are	    #
#	free to use, modify, distribute, publish, etc. provided attribution.  Please forward 	    #
#	bugs, complaints, comments, and useful changes to: jgill@latte.harvard.edu.  		    #
#											    	    #
#####################################################################################################

###################################### SETUP ########################################################
scotland.housing <- as.matrix(read.table("scotland_housing.txt",header=T))
scotland.population <- as.matrix(read.table("scotland_population.txt",header=T))
scotland.education <- as.matrix(read.table("scotland_education.txt",header=T))
scotland.births <- as.matrix(read.table("scotland_births.txt",header=T))
scotland.economics <- as.matrix(read.table("scotland_economics.txt",header=T))
scotland.gdp <- c(13566,13566,9611,9483,9265,9555,9611,9483,9483,12656,9483,12656,8298,9265,8314,9483,
	8298,9483,12656,13566,9483,9483,8298,9611,9483,9033,8298,9483,9483,9265,9483,12656)
scotland.vote1 <- as.matrix(read.table("scotland_devolution1.txt",header=T))
scotland.vote2 <- as.matrix(read.table("scotland_devolution2.txt",header=T))
scotland.df <- data.frame(cbind(scotland.housing,scotland.population,scotland.education,scotland.births,
		scotland.economics,scotland.gdp,scotland.vote1[,2],scotland.vote2[,2]))
dimnames(scotland.df)[[2]][40] <- "GDP"
dimnames(scotland.df)[[2]][41] <- "PerYesParl"
dimnames(scotland.df)[[2]][42] <- "PerYesTax"
write.table(scotland.df, file="export/homescotvote.dat")
attach(scotland.df)
###################################### LOOK AT OUTCOME VARIABLE #####################################
postscript("tax1.ps",paper="letter",horizontal=T)
par(mfrow=c(4,4))
for(i in 1:16)  {
	plot(scotland.df[,i],PerYes,xlab=dimnames(scotland.df)[[2]][i])
	lines(lowess(scotland.df[,i],PerYes))
}
dev.off()
postscript("tax2.ps",paper="letter",horizontal=T)
par(mfrow=c(4,4))
for(i in 17:32)  {
	plot(scotland.df[,i],PerYes,xlab=dimnames(scotland.df)[[2]][i])
	if(sum(is.na(scotland.df[,i])) == 0)
		lines(lowess(scotland.df[,i],PerYes))
}
dev.off()
postscript("tax3.ps",paper="letter",horizontal=T)
par(mfrow=c(4,4))
for(i in 33:44)  {
	plot(scotland.df[,i],PerYes,xlab=dimnames(scotland.df)[[2]][i])
	if(sum(is.na(scotland.df[,i])) == 0)
		lines(lowess(scotland.df[,i],PerYes))
}
dev.off()
###################################### DEVELOP GLM ##################################################
scottish.vote.glm <-  glm((PerYesTax/100) ~ CouncilTax*PerClaimantFemale+StdMortalityRatio+Active+Percentage5to15,
        		family=Gamma,data=scotland.df)
vote.sum <- summary.glm(scottish.vote.glm)
min(abs( eigen(-glm.vc(scottish.vote.glm2))$values^(-1) ))

round( cbind( 
	vote.sum$coefficients[,1], vote.sum$coefficients[,2],
	vote.sum$coefficients[,1] - vote.sum$coefficients[,2]*qt(0.975,vote.sum$df[2]),
	vote.sum$coefficients[,1] + vote.sum$coefficients[,2]*qt(0.975,vote.sum$df[2])	
),7)
#####################################################################################################
