#####################################################################################################
#												    #
#	This archive is part of the free distribution of data and statistical software code for	    #
#	"Generalized Linear Models: A Unified Approach", Jeff Gill, Sage QASS Series.  You are	    #
#	free to use, modify, distribute, publish, etc. provided attribution.  Please forward 	    #
#	bugs, complaints, comments, and useful changes to: jgill@latte.harvard.edu.  		    #
#											    	    #
#####################################################################################################

###################################### SETUP ########################################################
star.data <- as.matrix(read.table("star.bi.dat",header=F))

star.factors <- data.frame( LOWINC=star.data[,2], PERASIAN=star.data[,3], PERBLACK=star.data[,4],
	PERHISP=star.data[,5], PERMINTE=star.data[,6], AVYRSEXP=star.data[,7], AVSAL=star.data[,8],
	PERSPEN=star.data[,9], PTRATIO=star.data[,10], PCTAF=star.data[,11], PCTCHRT=star.data[,12],
	PCTYRRND=star.data[,13], READTOT=star.data[,14], PR50RD=star.data[,15],
	MATHTOT=star.data[,16], PR50M=star.data[,17] ) 
attach(star.factors)
#####################################################################################################

###################################### MODEL ########################################################
star.logit.fit <- glm(cbind(PR50M,MATHTOT-PR50M) ~ LOWINC + PERASIAN + PERBLACK + PERHISP +
	PERMINTE * AVYRSEXP * AVSAL + PERSPEN * PTRATIO * PCTAF + PCTCHRT + PCTYRRND,
        family=binomial(link=logit),data=star.factors)
summary.glm(star.logit.fit)
anova(star.logit.fit)
#####################################################################################################

###################################### RESIDUALS ANALYSIS ###########################################
star.pears <- resid(star.logit.fit,type="pearson")
star.pears%*%star.pears
star.mat <- cbind(rep(1,nrow(star.factors)),as.matrix(star.factors[1:12]))
star.mu <- predict.glm(star.logit.fit,type="response")
star.y <- PR50M/MATHTOT
star.n <- length(star.y)

sum(resid(star.logit.fit,type="response")) 
sum(resid(star.logit.fit,type="pearson")) 

star.phi <- cs.string
###################################### ADJUSTED DEVIANCES ###########################################
2*sum(  PR50M*log(star.y/star.mu) + (MATHTOT-PR50M)*log((MATHTOT-PR50M)/(MATHTOT-star.mu*MATHTOT)) )
[1] 4078.765

mean(PR50M)	[1] 357.8152
PR50M.adj <- PR50M
for (i in 1:length(PR50M.adj))  {
	if (PR50M.adj[i] > mean(PR50M)) PR50M.adj[i] <- PR50M.adj[i] - 0.5
	if (PR50M.adj[i] < mean(PR50M)) PR50M.adj[i] <- PR50M.adj[i] + 0.5
}

2*sum(  PR50M.adj*log(PR50M.adj/(star.mu*MATHTOT)) 
        + (MATHTOT-PR50M.adj)*log((MATHTOT-PR50M.adj)/(MATHTOT-star.mu*MATHTOT)) )
[1] 4054.928

range(abs( eigen(-glm.vc(star.logit.fit))$values^(-1) ))
###################################### FIGURE 1: MODEL FIT DIAGNOSTICS ##############################
postscript("glm.fig1.ps")#,width=3,height=3)
par(mfrow=c(1,3),mar=c(8,8,6,2))

plot(star.mu,star.y,xlab="",ylab="")
mtext(side=1,outer=F,"Fitted Values",line=6,cex=1.5)
mtext(side=2,outer=F,"Observed Values",line=4,cex=1.3)
mtext(side=3,outer=F,"Model Fit Plot",line=3,cex=1.4)
abline(lm(star.y~star.mu)$coefficients)

plot(fitted(star.logit.fit),resid(star.logit.fit,type="pearson"),xlab="",ylab="")
mtext(side=1,outer=F,"Fitted Values",line=6,cex=1.5)
mtext(side=2,outer=F,"Pearson Residuals",line=4,cex=1.3)
mtext(side=3,outer=F,"Residual Dependence Plot",line=3,cex=1.4)
abline(0,0)

qqnorm(resid(star.logit.fit,type="deviance"),main="",xlab="",ylab="")
mtext(side=1,outer=F,"Quantiles of N(0,1)",line=6,cex=1.5)
mtext(side=2,outer=F,"Deviance Residual Quantiles",line=4,cex=1.3)
mtext(side=3,outer=F,"Normal-Quantile Plot",line=3,cex=1.4)
abline(-0.3,3.5)
dev.off()
#####################################################################################################
